/* continuation.cc                                             -*- C++ -*-
 * Morgan Deters <mdeters@morgandeters.com>
 * September 2003
 *
 * This implements continuations for gcc on x86 platforms with
 * glibc 2.3.2 (mileage may vary on other libc versions).
 *
 * THIS CODE USES GLOBAL VARIABLES AND IS EXPLICITLY NOT THREAD-SAFE.
 * It was written to support a user-space threading library
 * implementation running in a single kernel thread.  In that context,
 * its thread-safety is not an issue since all continuation activity
 * will be atomic with respect to that kernel thread, and none of the
 * user-space threads it manages will access the continuation library
 * (or its global state) directly.
 *
 * This file is part of the RMA Templates project.  For additional
 * information and build instructions, please see the README file in
 * the top-level source directory.
 *
 * Copyright (c) 2003 MORGAN DETERS
 * All rights reserved.
 *
 * This software is open-source software and may be redistributed and
 * modified under the terms of an amended BSD License.  For the terms
 * of the license, see the file COPYING in the top-level directory.
 * If this file is missing for some reason, please obtain the latest
 * release of these sources from:
 *   http://www.cs.wustl.edu/~mdeters/projects/middleware/#rma_templates
 */

#include "continuation.h"

using namespace std;

using continuation::continuation_t;

continuation_t *continuation_t::cc_ = 0;

void continuation_t::enter_cc(continuation_t* c) throw() {
  //struct rlimit rlim;
  //set_stack_limited_to(rlim, cc_->n_);
  cc_->e_.entry_ptr_(c);
}

void continuation_t::enter(continuation_t* c) throw() {
  //struct rlimit rlim;
  //set_stack_limited_to(rlim, cc_->n_);
  cc_->e_.entry_void_();
}

int continuation_t::operator()(int i) throw() {
  //DEBUG(("op()(%p,%p)\n", cc_, this));

  arg_ = i;

  if(this == cc_)
    return i;

  //struct rlimit rlim;

  if(!sigsetjmp(cc_->self_, 1)) {
    cc_ = this;
    //set_stack_unlimited(rlim);
    siglongjmp(self_, 1);
  }

  //set_stack_limited_to(rlim, cc_->n_);

  return cc_->arg_;
  //int: compute like a capuchin monkey! Do you hear me, slave? 0101110101!
  //That's it(scream) like a parakeet! 0010100?  int: flashing lights, a spool of magnetic tape-1.
}

bool continuation_t::save() throw() {
  //struct rlimit rlim;
  if(!sigsetjmp(self_, 1))
    return false;
  //set_stack_limited_to(rlim, cc_->n_);
  return true;
}

void continuation_t::jump(int i) throw() {
  //struct rlimit rlim;
  arg_ = i;
  cc_ = this;
  //set_stack_unlimited(rlim);
  siglongjmp(self_, 1);
}
