/* examples/infeasible.cc
 * Simple example of an infeasible taskset.  This program will not
 * compile, as it contains a static check for feasibility of the task
 * set.  Specifically, a type error is raised indicating the fault.
 *
 * This file is part of the RMA Templates project.  For additional
 * information and build instructions, please see the README file in
 * the top-level source directory.
 *
 * Copyright (c) 2003, MORGAN DETERS, CHRISTOPHER GILL, and RON CYTRON
 * All rights reserved.
 *
 * This software is open-source software and may be redistributed and
 * modified under the terms of an amended BSD License.  For the terms
 * of the license, see the file COPYING in the top-level directory.
 * If this file is missing for some reason, please obtain the latest
 * release of these sources from:
 *   http://www.cs.wustl.edu/~mdeters/projects/middleware/#rma_templates
 */

#include <iostream>

#include <loki/static_check.h>

#include "rma.h"
#include "schedule.h"

// define two tasks, taskA and taskB

struct taskA {
  enum { cost       = 8,
         period     = 10,
         release    = 1,
         /* ... */         };
};

struct taskB {
  enum { cost       = 5,
         period     = 15,
         release    = 4,
         /* ... */         };
};

// define a taskset
typedef TYPELIST_2(taskA, taskB) my_tasks;

// define a schedule - the second template parameter (the scheduler)
// isn't specified, so we get a Null_Scheduler---useless for
// scheduling, but fine if we just want to determine the feasibility
// of a taskset
typedef RMA::Schedule<my_tasks> my_schedule;

int main() {
  // This static check fails for this taskset; the schedule is infeasible.
  // The program will fail to compile.
  STATIC_CHECK(my_schedule::feasible, Schedule_Infeasible);

  // Since we didn't name a scheduler in the RMA::Schedule<>
  // instantiation, we got the Null_Scheduler, and trying to schedule
  // our taskset will give a compile-time error:
  my_schedule::schedule();

  std::cout << "PLEASE NOTE:" << std::endl
            << "This program is an example of the compile-time features "
               "of RMA Templates." << std::endl
            << "As you can see, it does nothing interesting "
               "at runtime." << std::endl;

  return 0;
}
