/* examples/simple_tasks.cc
 * Simple example of scheduling an RMA Template schedule.
 *
 * This file is part of the RMA Templates project.  For additional
 * information and build instructions, please see the README file in
 * the top-level source directory.
 *
 * Copyright (c) 2003, MORGAN DETERS, CHRISTOPHER GILL, and RON CYTRON
 * All rights reserved.
 *
 * This software is open-source software and may be redistributed and
 * modified under the terms of an amended BSD License.  For the terms
 * of the license, see the file COPYING in the top-level directory.
 * If this file is missing for some reason, please obtain the latest
 * release of these sources from:
 *   http://www.cs.wustl.edu/~mdeters/projects/middleware/#rma_templates
 */

#include <iostream>

#include <loki/static_check.h>

#include "rma.h"
#include "schedule.h"
#include "simple_scheduler.h"

// define two tasks, taskA and taskB

struct taskA {
  enum { cost       = 5,
         period     = 10,
         release    = 1,
         /* ... */         };

  static void entry(void) {
    std::cout << "in taskA..." << std::endl;
  }
};

struct taskB {
  enum { cost       = 5,
         period     = 20,
         release    = 4,
         /* ... */         };

  static void entry(void) {
    std::cout << "in taskB..." << std::endl;
  }
};

// define a taskset
typedef TYPELIST_2(taskA, taskB) my_tasks;

// define a schedule - scheduling done with Simple_Scheduler
typedef RMA::Schedule<my_tasks, RMA::Simple_Scheduler> my_schedule;

int main() {
  // This static check succeeds for this taskset; the schedule is feasible.
  // The program will compile normally.
  STATIC_CHECK(my_schedule::feasible, Schedule_Infeasible);

  // schedule this with Simple_Scheduler
  my_schedule::schedule();

  return 0;
}
