/* print_scheduler.h                                           -*- C++ -*-
 * Print scheduler---prints the major cycle but doesn't schedule anything.
 *
 * This file is part of the RMA Templates project.  For additional
 * information and build instructions, please see the README file in
 * the top-level source directory.
 *
 * Copyright (c) 2003, MORGAN DETERS, CHRISTOPHER GILL, and RON CYTRON
 * All rights reserved.
 *
 * This software is open-source software and may be redistributed and
 * modified under the terms of an amended BSD License.  For the terms
 * of the license, see the file COPYING in the top-level directory.
 * If this file is missing for some reason, please obtain the latest
 * release of these sources from:
 *   http://www.cs.wustl.edu/~mdeters/projects/middleware/#rma_templates
 */

#include <iostream>
#include <iomanip>

#include <loki/NullType.h>
#include <loki/Typelist.h>

namespace RMA {
  template <class Type> struct Print_Scheduler_Type_String;
  template <unsigned t, class Task>
  struct Print_Scheduler_Type_String<Release<t, Task> > {
    static const char type_string[];
  };
  template <unsigned t, class Task>
  struct Print_Scheduler_Type_String<Suspend<t, Task> > {
    static const char type_string[];
  };
  template <unsigned t, class Task>
  struct Print_Scheduler_Type_String<Resume<t, Task> > {
    static const char type_string[];
  };
  template <unsigned t, class Task>
  struct Print_Scheduler_Type_String<Finish<t, Task> > {
    static const char type_string[];
  };
  template <unsigned t, unsigned n>
  struct Print_Scheduler_Type_String<Idle<t, n> > {
    static const char type_string[];
  };

  template <unsigned t, class Task> const char Print_Scheduler_Type_String<Release<t, Task> >::type_string[] = "Release";
  template <unsigned t, class Task> const char Print_Scheduler_Type_String<Suspend<t, Task> >::type_string[] = "Suspend";
  template <unsigned t, class Task> const char Print_Scheduler_Type_String<Resume<t, Task> >::type_string[] = "Resume";
  template <unsigned t, class Task> const char Print_Scheduler_Type_String<Finish<t, Task> >::type_string[] = "Finish";
  template <unsigned t, unsigned n> const char Print_Scheduler_Type_String<Idle<t, n> >::type_string[] = "Idle";

  template <class Cycle> struct Print_Scheduler_printer;
  template <class E, class Tail>
  struct Print_Scheduler_printer<Loki::Typelist<E, Tail> > {
    static void print(std::ostream& out) {
      std::cout << std::setw(5) << E::t << " "
                << std::setw(8) << Print_Scheduler_Type_String<E>::type_string
                << ": \""
                << std::setw(30) << typeid(E::Task).name()
                << "\" (c="
                << std::setw(5) << E::Task::cost << ", t="
                << std::setw(5) << E::Task::period << ", r="
                << std::setw(5) << E::Task::release << ")" << std::endl;
      Print_Scheduler_printer<Tail>::print(out);
    }
  };
  template <>
  struct Print_Scheduler_printer<Loki::NullType> {
    static void print(std::ostream& out) {
      // base of recursion
    }
  };

  template <class Cycle, unsigned lcm>
  struct Print_Scheduler {
    static void schedule(void) {
      std::cout << "Major Cycle (length=" << lcm
                << ", # events=" << Loki::TL::Length<Cycle>::value << ")"
                << std::endl;
      Print_Scheduler_printer<Cycle>::print(std::cout);
      std::cout << "-------------" << std::endl;
    }
  };
}
