/* simple_scheduler.h                                          -*- C++ -*-
 * Simple non-threading scheduler for RMA Template schedules
 * THIS SCHEDULER IS NOT FULL-FEATURED
 *
 * This file is part of the RMA Templates project.  For additional
 * information and build instructions, please see the README file in
 * the top-level source directory.
 *
 * Copyright (c) 2003, MORGAN DETERS, CHRISTOPHER GILL, and RON CYTRON
 * All rights reserved.
 *
 * This software is open-source software and may be redistributed and
 * modified under the terms of an amended BSD License.  For the terms
 * of the license, see the file COPYING in the top-level directory.
 * If this file is missing for some reason, please obtain the latest
 * release of these sources from:
 *   http://www.cs.wustl.edu/~mdeters/projects/middleware/#rma_templates
 */

#include <loki/NullType.h>
#include <loki/Typelist.h>

#include <loki/static_check.h>

// #define DEBUG(x) std::cout << x << std::endl
#define DEBUG(x)

namespace RMA {
  template <class TaskSet>
  struct Simple_Scheduler_Expand;

  template <unsigned t, class Task, class Tail>
  struct Simple_Scheduler_Expand<Loki::Typelist<Release<t, Task>, Tail> > {
    static void schedule(void) {
      DEBUG(t << " RELEASE");
      Task::entry();
      Simple_Scheduler_Expand<Tail>::schedule();
    }
  };

  template <unsigned t, class Task, class Tail>
  struct Simple_Scheduler_Expand<Loki::Typelist<Suspend<t, Task>, Tail> > {
    static void schedule(void) {
      STATIC_CHECK(false, Simple_Scheduler_Cannot_Suspend_Tasks);
      Simple_Scheduler_Expand<Tail>::schedule();
    }
  };

  template <unsigned t, unsigned n, class Tail>
  struct Simple_Scheduler_Expand<Loki::Typelist<Idle<t, n>, Tail> > {
    static void schedule(void) {
      DEBUG(t << " IDLE(" << n << ')');
      Simple_Scheduler_Expand<Tail>::schedule();
    }
  };

  template <class E, class Tail>
  struct Simple_Scheduler_Expand<Loki::Typelist<E, Tail> > {
    static void schedule(void) {
      Simple_Scheduler_Expand<Tail>::schedule();
    }
  };

  template <>
  struct Simple_Scheduler_Expand<Loki::NullType> {
    static void schedule(void) {
    }
  };

  template <class TaskSet, unsigned lcm>
  struct Simple_Scheduler {
    static void schedule(void) {
      for(;;) {
        DEBUG("MAJOR CYCLE BEGINNING lcm = " << lcm);
        Simple_Scheduler_Expand<TaskSet>::schedule();
      }
    }
  };
}
